@@ -33,8 +33,6 @@
 #include <string.h>
 #include <limits.h>
 
-#include "slim_internal.h"
-
 #ifndef __GNUC__
 #define __inline
 #endif
@@ -45,7 +43,9 @@
 #define INLINE
 #endif
 
+#undef MIN
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
+#undef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
 
 /* C89 has implementation-defined behavior for % with negative operands.
@@ -66,7 +66,6 @@ typedef struct _FbPoint {
 
 typedef unsigned int	Mask;
 
-
 #define GXcopy		0x3
 #define GXor		0x7
 #define ClipByChildren  0
@@ -75,7 +74,6 @@ typedef unsigned int	Mask;
 #define CPClipMask      (1 << 6)
 #define CPLastBit       11
 
-
 /* Define any names that the server code will be expecting in
  * terms of libpixman names. */
 
@@ -116,7 +114,6 @@ typedef pixman_triangle_t	xTriangle;
 #  define BITMAP_BIT_ORDER LSBFirst
 #endif
 
-
 #define MAXSHORT SHRT_MAX
 #define MINSHORT SHRT_MIN
 
@@ -124,8 +121,6 @@ typedef pixman_triangle_t	xTriangle;
 #include "picture.h"
 */
 
-
-
 #include "pixman.h"
 
 /* XXX: Most of this file is straight from fb.h and I imagine we can
@@ -136,7 +131,7 @@ typedef pixman_triangle_t	xTriangle;
 #define FB_HALFUNIT (1 << (FB_SHIFT-1))
 #define FB_MASK	    (FB_UNIT - 1)
 #define FB_ALLONES  ((pixman_bits_t) -1)
-    
+
 /* whether to bother to include 24bpp support */
 #ifndef ICNO24BIT
 #define FB_24BIT
@@ -157,20 +152,18 @@ typedef pixman_triangle_t	xTriangle;
 #define FB_STIP_UNIT	(1 << FB_STIP_SHIFT)
 #define FB_STIP_MASK	(FB_STIP_UNIT - 1)
 #define FB_STIP_ALLONES	((FbStip) -1)
-    
+
 #define FB_STIP_ODDSTRIDE(s)	(((s) & (FB_MASK >> FB_STIP_SHIFT)) != 0)
 #define FB_STIP_ODDPTR(p)	((((long) (p)) & (FB_MASK >> 3)) != 0)
-    
+
 #define FbStipStrideToBitsStride(s) (((s) >> (FB_SHIFT - FB_STIP_SHIFT)))
 #define FbBitsStrideToStipStride(s) (((s) << (FB_SHIFT - FB_STIP_SHIFT)))
-    
-#define FbFullMask(n)   ((n) == FB_UNIT ? FB_ALLONES : ((((FbBits) 1) << n) - 1))
 
+#define FbFullMask(n)   ((n) == FB_UNIT ? FB_ALLONES : ((((FbBits) 1) << n) - 1))
 
 typedef uint32_t	    FbStip;
 typedef int		    FbStride;
 
-
 #ifdef FB_DEBUG
 extern void fbValidateDrawable(DrawablePtr d);
 extern void fbInitializeDrawable(DrawablePtr d);
@@ -223,7 +216,6 @@ extern void fbSetBits (FbStip *bits, int stride, FbStip data);
 #define FbStipMask(x,w)	(FbStipRight(FB_STIP_ALLONES,(x) & FB_STIP_MASK) & \
 			 FbStipLeft(FB_STIP_ALLONES,(FB_STIP_UNIT - ((x)+(w))) & FB_STIP_MASK))
 
-
 #define FbMaskBits(x,w,l,n,r) { \
     n = (w); \
     r = FbRightMask((x)+n); \
@@ -441,7 +433,6 @@ extern void fbSetBits (FbStip *bits, int stride, FbStip data);
     } \
 }
 
-
 #define FbDoRightMaskByteRRop(dst,rb,r,and,xor) { \
     switch (rb) { \
     case 1: \
@@ -590,21 +581,21 @@ extern void fbSetBits (FbStip *bits, int stride, FbStip data);
  * icblt.c
  */
 pixman_private void
-fbBlt (pixman_bits_t   *src, 
+fbBlt (pixman_bits_t   *src,
        FbStride	srcStride,
        int	srcX,
-       
+
        FbBits   *dst,
        FbStride dstStride,
        int	dstX,
-       
-       int	width, 
+
+       int	width,
        int	height,
-       
+
        int	alu,
        FbBits	pm,
        int	bpp,
-       
+
        Bool	reverse,
        Bool	upsidedown);
 
@@ -617,7 +608,7 @@ fbBlt24 (pixman_bits_t	    *srcLine,
 	 FbStride   dstStride,
 	 int	    dstX,
 
-	 int	    width, 
+	 int	    width,
 	 int	    height,
 
 	 int	    alu,
@@ -625,23 +616,23 @@ fbBlt24 (pixman_bits_t	    *srcLine,
 
 	 Bool	    reverse,
 	 Bool	    upsidedown);
-    
+
 pixman_private void
 fbBltStip (FbStip   *src,
 	   FbStride srcStride,	    /* in FbStip units, not FbBits units */
 	   int	    srcX,
-	   
+
 	   FbStip   *dst,
 	   FbStride dstStride,	    /* in FbStip units, not FbBits units */
 	   int	    dstX,
 
-	   int	    width, 
+	   int	    width,
 	   int	    height,
 
 	   int	    alu,
 	   FbBits   pm,
 	   int	    bpp);
-    
+
 /*
  * icbltone.c
  */
@@ -661,7 +652,7 @@ fbBltOne (FbStip   *src,
 	  FbBits   fbxor,
 	  FbBits   bgand,
 	  FbBits   bgxor);
- 
+
 #ifdef FB_24BIT
 pixman_private void
 fbBltOne24 (FbStip    *src,
@@ -820,7 +811,7 @@ fbRasterizeTrapezoid (pixman_image_t		*pMask,
 # endif
 #else
 # define ICINT_NEED_IC_ONES
-int
+pixman_private int
 _FbOnes(unsigned long mask);
 #endif
 
@@ -859,18 +850,6 @@ pixman_private int
 pixman_transform_point (pixman_transform_t	*transform,
 		  pixman_vector_t	*vector);
 
-/* Avoid unnessecary PLT entries.  */
-
-slim_hidden_proto(pixman_image_create)
-slim_hidden_proto(pixman_color_to_pixel)
-slim_hidden_proto(pixman_format_init)
-slim_hidden_proto(pixman_image_destroy)
-slim_hidden_proto(pixman_fill_rectangles)
-slim_hidden_proto(pixman_image_set_component_alpha)
-slim_hidden_proto(pixman_image_set_repeat)
-slim_hidden_proto(pixman_composite)
-
-
 #include "icrop.h"
 
 /* XXX: For now, I'm just wholesale pasting Xserver/render/picture.h here: */
@@ -956,7 +935,7 @@ typedef struct _PictFormat	*PictFormatPtr;
 #define PICT_b1g2r1	PICT_FORMAT(4,PICT_TYPE_ABGR,0,1,2,1)
 #define PICT_a1r1g1b1	PICT_FORMAT(4,PICT_TYPE_ARGB,1,1,1,1)
 #define PICT_a1b1g1r1	PICT_FORMAT(4,PICT_TYPE_ABGR,1,1,1,1)
-				    
+
 #define PICT_c4		PICT_FORMAT(4,PICT_TYPE_COLOR,0,0,0,0)
 #define PICT_g4		PICT_FORMAT(4,PICT_TYPE_GRAY,0,0,0,0)
 
@@ -966,7 +945,7 @@ typedef struct _PictFormat	*PictFormatPtr;
 #define PICT_g1		PICT_FORMAT(1,PICT_TYPE_GRAY,0,0,0,0)
 
 /*
- * For dynamic indexed visuals (GrayScale and PseudoColor), these control the 
+ * For dynamic indexed visuals (GrayScale and PseudoColor), these control the
  * selection of colors allocated for drawing to Pictures.  The default
  * policy depends on the size of the colormap:
  *
@@ -997,7 +976,7 @@ typedef struct _PictFormat	*PictFormatPtr;
  * 4x4x4 cube allocates another two and nine more are allocated to fill
  * in the 13 levels.  When the 4x4x4 cube is not allocated, a total of
  * 11 cells are allocated.
- */   
+ */
 
 #define PictureCmapPolicyInvalid    -1
 #define PictureCmapPolicyDefault    0
@@ -1006,7 +985,7 @@ typedef struct _PictFormat	*PictFormatPtr;
 #define PictureCmapPolicyColor	    3
 #define PictureCmapPolicyAll	    4
 
-extern int PictureCmapPolicy pixman_private;
+extern pixman_private int PictureCmapPolicy;
 
 int	PictureParseCmapPolicy (const char *name);
 
@@ -1037,8 +1016,8 @@ typedef uint32_t		xFixed_1_16;
 typedef int32_t		xFixed_16_16;
 
 /*
- * An unadorned "xFixed" is the same as xFixed_16_16, 
- * (since it's quite common in the code) 
+ * An unadorned "xFixed" is the same as xFixed_16_16,
+ * (since it's quite common in the code)
  */
 typedef	xFixed_16_16	xFixed;
 #define XFIXED_BITS	16
@@ -1078,7 +1057,6 @@ typedef	xFixed_16_16	xFixed;
 
 #endif /* _PICTURE_H_ */
 
-
 /* Macros needed by fbpict.c */
 
 #define cvt8888to0565(s)    ((((s) >> 3) & 0x001f) | \
