@@ -101,17 +101,17 @@ _cairo_beos_surface_create_internal (BView*   view,
 				     bool     owns_bitmap_view = false);
 
 static BRect
-_cairo_rect_to_brect (const cairo_rectangle_fixed_t* rect)
+_cairo_rect_to_brect (const cairo_rectangle_int16_t* rect)
 {
     // A BRect is one pixel wider than you'd think
     return BRect(rect->x, rect->y, rect->x + rect->width - 1,
 	    	 rect->y + rect->height - 1);
 }
 
-static cairo_rectangle_fixed_t
+static cairo_rectangle_int16_t
 _brect_to_cairo_rect (const BRect& rect)
 {
-    cairo_rectangle_fixed_t retval;
+    cairo_rectangle_int16_t retval;
     retval.x = int(rect.left + 0.5);
     retval.y = int(rect.top + 0.5);
     retval.width = rect.IntegerWidth() + 1;
@@ -550,9 +550,9 @@ _cairo_beos_surface_release_source_image (void                  *abstract_surfac
 
 static cairo_status_t
 _cairo_beos_surface_acquire_dest_image (void			 *abstract_surface,
-                                        cairo_rectangle_fixed_t	 *interest_rect,
+                                        cairo_rectangle_int16_t	 *interest_rect,
                                         cairo_image_surface_t	**image_out,
-                                        cairo_rectangle_fixed_t	 *image_rect,
+                                        cairo_rectangle_int16_t	 *image_rect,
                                         void			**image_extra)
 {
     cairo_beos_surface_t *surface = reinterpret_cast<cairo_beos_surface_t*>(
@@ -615,9 +615,9 @@ _cairo_beos_surface_acquire_dest_image (void			 *abstract_surface,
 
 static void
 _cairo_beos_surface_release_dest_image (void			*abstract_surface,
-                                        cairo_rectangle_fixed_t	*intersect_rect,
+                                        cairo_rectangle_int16_t	*intersect_rect,
                                         cairo_image_surface_t	*image,
-                                        cairo_rectangle_fixed_t	*image_rect,
+                                        cairo_rectangle_int16_t	*image_rect,
                                         void			*image_extra)
 {
     fprintf(stderr, "Fallback drawing\n");
@@ -768,7 +768,7 @@ _cairo_beos_surface_composite (cairo_operator_t		op,
 
 static void
 _cairo_beos_surface_fill_rectangle (cairo_beos_surface_t	*surface,
-				    cairo_rectangle_fixed_t	*rect)
+				    cairo_rectangle_int16_t	*rect)
 {
     BRect brect(_cairo_rect_to_brect(rect));
     surface->view->FillRect(brect);
@@ -778,7 +778,7 @@ static cairo_int_status_t
 _cairo_beos_surface_fill_rectangles (void			*abstract_surface,
 				     cairo_operator_t		 op,
 				     const cairo_color_t	*color,
-				     cairo_rectangle_fixed_t	*rects,
+				     cairo_rectangle_int16_t	*rects,
 				     int			 num_rects)
 {
     fprintf(stderr, "Drawing %i rectangles\n", num_rects);
@@ -863,7 +863,7 @@ _cairo_beos_surface_set_clip_region (void              *abstract_surface,
 
 static cairo_int_status_t
 _cairo_beos_surface_get_extents (void				*abstract_surface,
-				 cairo_rectangle_fixed_t	*rectangle)
+				 cairo_rectangle_int16_t	*rectangle)
 {
     cairo_beos_surface_t *surface = reinterpret_cast<cairo_beos_surface_t*>(
 							abstract_surface);
@@ -926,7 +926,10 @@ _cairo_beos_surface_create_internal (BView*   view,
         return const_cast<cairo_surface_t*>(&_cairo_surface_nil);
     }
 
-    _cairo_surface_init(&surface->base, &cairo_beos_surface_backend);
+    cairo_content_t content = CAIRO_CONTENT_COLOR;
+    if (bmp && (bmp->ColorSpace() == B_RGBA32 || bmp->ColorSpace() == B_RGBA15))
+	content = CAIRO_CONTENT_COLOR_ALPHA;
+    _cairo_surface_init(&surface->base, &cairo_beos_surface_backend, content);
 
     surface->view = view;
     surface->bitmap = bmp;
