@@ -31,7 +31,7 @@
 
 const cairo_solid_pattern_t cairo_pattern_nil = {
     { CAIRO_PATTERN_TYPE_SOLID, 	/* type */
-      (unsigned int)-1,		/* ref_count */
+      CAIRO_REF_COUNT_INVALID,		/* ref_count */
       CAIRO_STATUS_NO_MEMORY,	/* status */
       { 1., 0., 0., 1., 0., 0., }, /* matrix */
       CAIRO_FILTER_DEFAULT,	/* filter */
@@ -40,47 +40,13 @@ const cairo_solid_pattern_t cairo_pattern_nil = {
 
 static const cairo_solid_pattern_t cairo_pattern_nil_null_pointer = {
     { CAIRO_PATTERN_TYPE_SOLID, 	/* type */
-      (unsigned int)-1,		/* ref_count */
+      CAIRO_REF_COUNT_INVALID,		/* ref_count */
       CAIRO_STATUS_NULL_POINTER,/* status */
       { 1., 0., 0., 1., 0., 0., }, /* matrix */
       CAIRO_FILTER_DEFAULT,	/* filter */
       CAIRO_EXTEND_GRADIENT_DEFAULT },	/* extend */
 };
 
-static const cairo_solid_pattern_t cairo_pattern_nil_file_not_found = {
-    { CAIRO_PATTERN_TYPE_SOLID, 	/* type */
-      (unsigned int)-1,		/* ref_count */
-      CAIRO_STATUS_FILE_NOT_FOUND, /* status */
-      { 1., 0., 0., 1., 0., 0., }, /* matrix */
-      CAIRO_FILTER_DEFAULT,	/* filter */
-      CAIRO_EXTEND_GRADIENT_DEFAULT },	/* extend */
-};
-
-static const cairo_solid_pattern_t cairo_pattern_nil_read_error = {
-    { CAIRO_PATTERN_TYPE_SOLID, 	/* type */
-      (unsigned int)-1,		/* ref_count */
-      CAIRO_STATUS_READ_ERROR,	/* status */
-      { 1., 0., 0., 1., 0., 0., }, /* matrix */
-      CAIRO_FILTER_DEFAULT,	/* filter */
-      CAIRO_EXTEND_GRADIENT_DEFAULT },	/* extend */
-};
-
-static const cairo_pattern_t *
-_cairo_pattern_nil_for_status (cairo_status_t status)
-{
-    switch (status) {
-    case CAIRO_STATUS_NULL_POINTER:
-	return &cairo_pattern_nil_null_pointer.base;
-    case CAIRO_STATUS_FILE_NOT_FOUND:
-	return &cairo_pattern_nil_file_not_found.base;
-    case CAIRO_STATUS_READ_ERROR:
-	return &cairo_pattern_nil_read_error.base;
-    default:
-    case CAIRO_STATUS_NO_MEMORY:
-	return &cairo_pattern_nil.base;
-    }
-}
-
 /**
  * _cairo_pattern_set_error:
  * @pattern: a pattern
@@ -293,6 +259,20 @@ _cairo_pattern_create_solid (const cairo_color_t *color)
     return &pattern->base;
 }
 
+static const cairo_pattern_t *
+_cairo_pattern_create_for_status (cairo_status_t status)
+{
+    cairo_pattern_t *pattern;
+
+    pattern = _cairo_pattern_create_solid (_cairo_stock_color (CAIRO_STOCK_BLACK));
+    if (cairo_pattern_status (pattern))
+	return pattern;
+
+    _cairo_pattern_set_error (pattern, status);
+
+    return pattern;
+}
+
 /**
  * cairo_pattern_create_rgb:
  * @red: red component of the color
@@ -331,6 +311,7 @@ cairo_pattern_create_rgb (double red, double green, double blue)
 
     return pattern;
 }
+slim_hidden_def (cairo_pattern_create_rgb);
 
 /**
  * cairo_pattern_create_rgba:
@@ -373,6 +354,7 @@ cairo_pattern_create_rgba (double red, double green, double blue,
 
     return pattern;
 }
+slim_hidden_def (cairo_pattern_create_rgba);
 
 /**
  * cairo_pattern_create_for_surface:
@@ -395,10 +377,10 @@ cairo_pattern_create_for_surface (cairo_surface_t *surface)
     cairo_surface_pattern_t *pattern;
 
     if (surface == NULL)
-	return (cairo_pattern_t*) _cairo_pattern_nil_for_status (CAIRO_STATUS_NULL_POINTER);
+	return (cairo_pattern_t*) &cairo_pattern_nil_null_pointer;
 
     if (surface->status)
-	return (cairo_pattern_t*) _cairo_pattern_nil_for_status (surface->status);
+	return (cairo_pattern_t*) _cairo_pattern_create_for_status (surface->status);
 
     pattern = malloc (sizeof (cairo_surface_pattern_t));
     if (pattern == NULL) {
@@ -410,6 +392,7 @@ cairo_pattern_create_for_surface (cairo_surface_t *surface)
 
     return &pattern->base;
 }
+slim_hidden_def (cairo_pattern_create_for_surface);
 
 /**
  * cairo_pattern_create_linear:
@@ -514,7 +497,7 @@ cairo_pattern_reference (cairo_pattern_t *pattern)
     if (pattern == NULL)
 	return NULL;
 
-    if (pattern->ref_count == (unsigned int)-1)
+    if (pattern->ref_count == CAIRO_REF_COUNT_INVALID)
 	return pattern;
 
     assert (pattern->ref_count > 0);
@@ -523,12 +506,16 @@ cairo_pattern_reference (cairo_pattern_t *pattern)
 
     return pattern;
 }
+slim_hidden_def (cairo_pattern_reference);
 
 /**
  * cairo_pattern_get_type:
  * @pattern: a #cairo_pattern_t
  *
- * Return value: The type of @pattern. See #cairo_pattern_type_t.
+ * This function returns the type a pattern.
+ * See #cairo_pattern_type_t for available types.
+ *
+ * Return value: The type of @pattern.
  *
  * Since: 1.2
  **/
@@ -537,6 +524,7 @@ cairo_pattern_get_type (cairo_pattern_t *pattern)
 {
     return pattern->type;
 }
+slim_hidden_def (cairo_pattern_get_type);
 
 /**
  * cairo_pattern_status:
@@ -553,6 +541,7 @@ cairo_pattern_status (cairo_pattern_t *pattern)
 {
     return pattern->status;
 }
+slim_hidden_def (cairo_pattern_status);
 
 /**
  * cairo_pattern_destroy:
@@ -568,7 +557,7 @@ cairo_pattern_destroy (cairo_pattern_t *pattern)
     if (pattern == NULL)
 	return;
 
-    if (pattern->ref_count == (unsigned int)-1)
+    if (pattern->ref_count == CAIRO_REF_COUNT_INVALID)
 	return;
 
     assert (pattern->ref_count > 0);
@@ -580,6 +569,7 @@ cairo_pattern_destroy (cairo_pattern_t *pattern)
     _cairo_pattern_fini (pattern);
     free (pattern);
 }
+slim_hidden_def (cairo_pattern_destroy);
 
 static void
 _cairo_pattern_add_color_stop (cairo_gradient_pattern_t *pattern,
@@ -591,7 +581,7 @@ _cairo_pattern_add_color_stop (cairo_gradient_pattern_t *pattern,
 {
     pixman_gradient_stop_t *new_stops;
     cairo_fixed_t	   x;
-    int			   i;
+    unsigned int	   i;
 
     new_stops = realloc (pattern->stops, (pattern->n_stops + 1) *
 			 sizeof (pixman_gradient_stop_t));
@@ -761,6 +751,7 @@ cairo_pattern_set_matrix (cairo_pattern_t      *pattern,
 
     pattern->matrix = *matrix;
 }
+slim_hidden_def (cairo_pattern_set_matrix);
 
 /**
  * cairo_pattern_get_matrix:
@@ -824,6 +815,7 @@ cairo_pattern_get_extend (cairo_pattern_t *pattern)
 {
     return pattern->extend;
 }
+slim_hidden_def (cairo_pattern_get_extend);
 
 void
 _cairo_pattern_transform (cairo_pattern_t	*pattern,
@@ -1079,7 +1071,7 @@ _cairo_pattern_is_opaque_solid (const cairo_pattern_t *pattern)
 static cairo_bool_t
 _gradient_is_opaque (const cairo_gradient_pattern_t *gradient)
 {
-    int i;
+    unsigned int i;
 
     for (i = 0; i < gradient->n_stops; i++)
 	if (! CAIRO_ALPHA_IS_OPAQUE (gradient->stops[i].color.alpha))
