@@ -40,18 +40,18 @@
 
 typedef struct {
     cairo_surface_t *dst;
-    cairo_rectangle_fixed_t extents;
+    cairo_rectangle_int16_t extents;
     cairo_image_surface_t *image;
-    cairo_rectangle_fixed_t image_rect;
+    cairo_rectangle_int16_t image_rect;
     void *image_extra;
 } fallback_state_t;
 
 /**
  * _fallback_init:
- * 
+ *
  * Acquire destination image surface needed for an image-based
  * fallback.
- * 
+ *
  * Return value: CAIRO_INT_STATUS_NOTHING_TO_DO if the extents are not
  * visible, CAIRO_STATUS_SUCCESS if some portion is visible and all
  * went well, or some error status otherwise.
@@ -70,7 +70,7 @@ _fallback_init (fallback_state_t *state,
     state->extents.y = y;
     state->extents.width = width;
     state->extents.height = height;
-    
+
     state->dst = dst;
 
     status = _cairo_surface_acquire_dest_image (dst, &state->extents,
@@ -104,7 +104,7 @@ typedef cairo_status_t (*cairo_draw_func_t) (void                          *clos
 					     cairo_surface_t               *dst,
 					     int                            dst_x,
 					     int                            dst_y,
-					     const cairo_rectangle_fixed_t *extents);
+					     const cairo_rectangle_int16_t *extents);
 
 static cairo_status_t
 _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
@@ -112,18 +112,18 @@ _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
 				cairo_draw_func_t             draw_func,
 				void                          *draw_closure,
 				cairo_surface_t               *dst,
-				const cairo_rectangle_fixed_t *extents)
+				const cairo_rectangle_int16_t *extents)
 {
     cairo_surface_t *mask;
     cairo_status_t status;
-    
+
     mask = cairo_surface_create_similar (dst,
 					 CAIRO_CONTENT_ALPHA,
 					 extents->width,
 					 extents->height);
     if (mask->status)
 	return CAIRO_STATUS_NO_MEMORY;
-    
+
     status = (*draw_func) (draw_closure, CAIRO_OPERATOR_ADD,
 			   NULL, mask,
 			   extents->x, extents->y,
@@ -138,7 +138,7 @@ _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
 						 extents);
     if (status)
 	goto CLEANUP_SURFACE;
-    
+
     _cairo_pattern_init_for_surface (mask_pattern, mask);
 
  CLEANUP_SURFACE:
@@ -157,7 +157,7 @@ _clip_and_composite_with_mask (cairo_clip_t                  *clip,
 			       cairo_draw_func_t              draw_func,
 			       void                          *draw_closure,
 			       cairo_surface_t               *dst,
-			       const cairo_rectangle_fixed_t *extents)
+			       const cairo_rectangle_int16_t *extents)
 {
     cairo_surface_pattern_t mask_pattern;
     cairo_status_t status;
@@ -168,7 +168,7 @@ _clip_and_composite_with_mask (cairo_clip_t                  *clip,
 					     dst, extents);
     if (status)
 	return status;
-	
+
     status = _cairo_surface_composite (op,
 				       src, &mask_pattern.base, dst,
 				       extents->x,     extents->y,
@@ -191,7 +191,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
 			     cairo_draw_func_t              draw_func,
 			     void                          *draw_closure,
 			     cairo_surface_t               *dst,
-			     const cairo_rectangle_fixed_t *extents)
+			     const cairo_rectangle_int16_t *extents)
 {
     cairo_surface_t *intermediate;
     cairo_surface_pattern_t dst_pattern;
@@ -237,7 +237,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
      */
     status = _cairo_clip_combine_to_surface (clip, CAIRO_OPERATOR_DEST_IN,
 					     intermediate,
-					     extents->x, extents->y,					     
+					     extents->x, extents->y,
 					     extents);
     if (status)
 	goto CLEANUP_SURFACE;
@@ -254,7 +254,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
     /* Now add the two results together
      */
     _cairo_pattern_init_for_surface (&intermediate_pattern, intermediate);
-    
+
     status = _cairo_surface_composite (CAIRO_OPERATOR_ADD,
 				       &intermediate_pattern.base, NULL, dst,
 				       0,              0,
@@ -263,7 +263,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
 				       extents->width, extents->height);
 
     _cairo_pattern_fini (&intermediate_pattern.base);
-    
+
  CLEANUP_SURFACE:
     cairo_surface_destroy (intermediate);
 
@@ -279,7 +279,7 @@ _clip_and_composite_source (cairo_clip_t                  *clip,
 			    cairo_draw_func_t              draw_func,
 			    void                          *draw_closure,
 			    cairo_surface_t               *dst,
-			    const cairo_rectangle_fixed_t *extents)
+			    const cairo_rectangle_int16_t *extents)
 {
     cairo_surface_pattern_t mask_pattern;
     cairo_status_t status;
@@ -292,7 +292,7 @@ _clip_and_composite_source (cairo_clip_t                  *clip,
 					     dst, extents);
     if (status)
 	return status;
-    
+
     /* Compute dest' = dest OUT (mask IN clip)
      */
     status = _cairo_surface_composite (CAIRO_OPERATOR_DEST_OUT,
@@ -320,7 +320,7 @@ _clip_and_composite_source (cairo_clip_t                  *clip,
 }
 
 static int
-_cairo_rectangle_empty (const cairo_rectangle_fixed_t *rect)
+_cairo_rectangle_empty (const cairo_rectangle_int16_t *rect)
 {
     return rect->width == 0 || rect->height == 0;
 }
@@ -343,7 +343,7 @@ _cairo_rectangle_empty (const cairo_rectangle_fixed_t *rect)
  *
  * @draw_func is to called to draw the mask; it will be called no more
  * than once.
- * 
+ *
  * Return value: %CAIRO_STATUS_SUCCESS if the drawing succeeded.
  **/
 static cairo_status_t
@@ -353,7 +353,7 @@ _clip_and_composite (cairo_clip_t                  *clip,
 		     cairo_draw_func_t              draw_func,
 		     void                          *draw_closure,
 		     cairo_surface_t               *dst,
-		     const cairo_rectangle_fixed_t *extents)
+		     const cairo_rectangle_int16_t *extents)
 {
     cairo_pattern_union_t solid_pattern;
     cairo_status_t status;
@@ -408,7 +408,7 @@ _composite_trap_region (cairo_clip_t            *clip,
 			cairo_operator_t         op,
 			cairo_surface_t         *dst,
 			pixman_region16_t       *trap_region,
-			cairo_rectangle_fixed_t *extents)
+			cairo_rectangle_int16_t *extents)
 {
     cairo_status_t status;
     cairo_pattern_union_t solid_pattern;
@@ -425,11 +425,11 @@ _composite_trap_region (cairo_clip_t            *clip,
 
     if (num_rects == 0)
 	return CAIRO_STATUS_SUCCESS;
-    
+
     if (num_rects > 1) {
       if (_cairo_surface_get_clip_mode (dst) != CAIRO_CLIP_MODE_REGION)
 	    return CAIRO_INT_STATUS_UNSUPPORTED;
-	
+
 	clip_serial = _cairo_surface_allocate_clip_serial (dst);
 	status = _cairo_surface_set_clip_region (dst,
 						 trap_region,
@@ -437,10 +437,10 @@ _composite_trap_region (cairo_clip_t            *clip,
 	if (status)
 	    return status;
     }
-    
+
     if (clip_surface)
 	_cairo_pattern_init_for_surface (&mask.surface, clip_surface);
-	
+
     status = _cairo_surface_composite (op,
 				       src,
 				       clip_surface ? &mask.base : NULL,
@@ -476,19 +476,19 @@ _composite_traps_draw_func (void                          *closure,
 			    cairo_surface_t               *dst,
 			    int                            dst_x,
 			    int                            dst_y,
-			    const cairo_rectangle_fixed_t *extents)
+			    const cairo_rectangle_int16_t *extents)
 {
     cairo_composite_traps_info_t *info = closure;
     cairo_pattern_union_t pattern;
     cairo_status_t status;
-    
+
     if (dst_x != 0 || dst_y != 0)
 	_cairo_traps_translate (info->traps, - dst_x, - dst_y);
 
     _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE);
     if (!src)
 	src = &pattern.base;
-    
+
     status = _cairo_surface_composite_trapezoids (op,
 						  src, dst, info->antialias,
 						  extents->x,         extents->y,
@@ -513,9 +513,9 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
     cairo_status_t status;
     pixman_region16_t *trap_region;
     pixman_region16_t *clear_region = NULL;
-    cairo_rectangle_fixed_t extents;
+    cairo_rectangle_int16_t extents;
     cairo_composite_traps_info_t traps_info;
-    
+
     if (traps->num_traps == 0)
 	return CAIRO_STATUS_SUCCESS;
 
@@ -538,20 +538,16 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
     else
     {
 	cairo_surface_t *clip_surface = clip ? clip->surface : NULL;
-	
+
 	status = _cairo_surface_get_extents (dst, &extents);
 	if (status)
 	    return status;
-	
+
 	if (trap_region && !clip_surface) {
 	    /* If we optimize drawing with an unbounded operator to
 	     * _cairo_surface_fill_rectangles() or to drawing with a
 	     * clip region, then we have an additional region to clear.
 	     */
-	    status = _cairo_surface_get_extents (dst, &extents);
-	    if (status)
-		return status;
-	    
 	    clear_region = _cairo_region_create_from_rectangle (&extents);
 	    if (clear_region == NULL)
 		return CAIRO_STATUS_NO_MEMORY;
@@ -559,12 +555,12 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 	    status = _cairo_clip_intersect_to_region (clip, clear_region);
 	    if (status)
 		return status;
-	    
+
 	    _cairo_region_extents_rectangle (clear_region,  &extents);
-	    
+
 	    if (pixman_region_subtract (clear_region, clear_region, trap_region) != PIXMAN_REGION_STATUS_SUCCESS)
 		return CAIRO_STATUS_NO_MEMORY;
-	    
+
 	    if (!pixman_region_not_empty (clear_region)) {
 		pixman_region_destroy (clear_region);
 		clear_region = NULL;
@@ -575,14 +571,14 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 		return status;
 	}
     }
-	
+
     if (status)
 	goto out;
-    
+
     if (trap_region)
     {
 	cairo_surface_t *clip_surface = clip ? clip->surface : NULL;
-	
+
 	if ((src->type == CAIRO_PATTERN_TYPE_SOLID || op == CAIRO_OPERATOR_CLEAR) &&
 	    !clip_surface)
 	{
@@ -592,7 +588,7 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 		color = CAIRO_COLOR_TRANSPARENT;
 	    else
 		color = &((cairo_solid_pattern_t *)src)->color;
-	  
+
 	    /* Solid rectangles special case */
 	    status = _cairo_surface_fill_region (dst, op, color, trap_region);
 	    if (!status && clear_region)
@@ -646,7 +642,7 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 	pixman_region_destroy (trap_region);
     if (clear_region)
 	pixman_region_destroy (clear_region);
-    
+
     return status;
 }
 
@@ -656,7 +652,7 @@ _cairo_surface_fallback_paint (cairo_surface_t	*surface,
 			       cairo_pattern_t	*source)
 {
     cairo_status_t status;
-    cairo_rectangle_fixed_t extents;
+    cairo_rectangle_int16_t extents;
     cairo_box_t box;
     cairo_traps_t traps;
 
@@ -665,14 +661,14 @@ _cairo_surface_fallback_paint (cairo_surface_t	*surface,
 	return status;
 
     if (_cairo_operator_bounded_by_source (op)) {
-	cairo_rectangle_fixed_t source_extents;
+	cairo_rectangle_int16_t source_extents;
 	status = _cairo_pattern_get_extents (source, &source_extents);
 	if (status)
 	    return status;
 
 	_cairo_rectangle_intersect (&extents, &source_extents);
     }
-    
+
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
     if (status)
 	return status;
@@ -685,7 +681,7 @@ _cairo_surface_fallback_paint (cairo_surface_t	*surface,
     status = _cairo_traps_init_box (&traps, &box);
     if (status)
 	return status;
-    
+
     _clip_and_composite_trapezoids (source,
 				    op,
 				    surface,
@@ -705,7 +701,7 @@ _cairo_surface_mask_draw_func (void                          *closure,
 			       cairo_surface_t               *dst,
 			       int                            dst_x,
 			       int                            dst_y,
-			       const cairo_rectangle_fixed_t *extents)
+			       const cairo_rectangle_int16_t *extents)
 {
     cairo_pattern_t *mask = closure;
 
@@ -732,7 +728,7 @@ _cairo_surface_fallback_mask (cairo_surface_t		*surface,
 			      cairo_pattern_t		*mask)
 {
     cairo_status_t status;
-    cairo_rectangle_fixed_t extents, source_extents, mask_extents;
+    cairo_rectangle_int16_t extents, source_extents, mask_extents;
 
     status = _cairo_surface_get_extents (surface, &extents);
     if (status)
@@ -745,7 +741,7 @@ _cairo_surface_fallback_mask (cairo_surface_t		*surface,
 
 	_cairo_rectangle_intersect (&extents, &source_extents);
     }
-    
+
     if (_cairo_operator_bounded_by_mask (op)) {
 	status = _cairo_pattern_get_extents (mask, &mask_extents);
 	if (status)
@@ -781,7 +777,7 @@ _cairo_surface_fallback_stroke (cairo_surface_t		*surface,
 {
     cairo_status_t status;
     cairo_traps_t traps;
-    
+
     _cairo_traps_init (&traps);
 
     status = _cairo_path_fixed_stroke_to_traps (path,
@@ -854,7 +850,7 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
 					  cairo_surface_t               *dst,
 					  int                            dst_x,
 					  int                            dst_y,
-					  const cairo_rectangle_fixed_t *extents)
+					  const cairo_rectangle_int16_t *extents)
 {
     cairo_show_glyphs_info_t *glyph_info = closure;
     cairo_pattern_union_t pattern;
@@ -866,7 +862,7 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
      */
     if (dst_x != 0 || dst_y != 0) {
 	int i;
-	
+
 	for (i = 0; i < glyph_info->num_glyphs; ++i)
 	{
 	    ((cairo_glyph_t *) glyph_info->glyphs)[i].x -= dst_x;
@@ -877,8 +873,8 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
     _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE);
     if (!src)
 	src = &pattern.base;
-    
-    status = _cairo_surface_old_show_glyphs (glyph_info->font, op, src, 
+
+    status = _cairo_surface_old_show_glyphs (glyph_info->font, op, src,
 					     dst,
 					     extents->x, extents->y,
 					     extents->x - dst_x,
@@ -890,9 +886,9 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
 
     if (status != CAIRO_INT_STATUS_UNSUPPORTED)
 	return status;
-    
-    status = _cairo_scaled_font_show_glyphs (glyph_info->font, 
-					     op, 
+
+    status = _cairo_scaled_font_show_glyphs (glyph_info->font,
+					     op,
 					     src, dst,
 					     extents->x,         extents->y,
 					     extents->x - dst_x,
@@ -916,7 +912,7 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 				     cairo_scaled_font_t	*scaled_font)
 {
     cairo_status_t status;
-    cairo_rectangle_fixed_t extents, glyph_extents;
+    cairo_rectangle_int16_t extents, glyph_extents;
     cairo_show_glyphs_info_t glyph_info;
 
     status = _cairo_surface_get_extents (surface, &extents);
@@ -925,23 +921,23 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 
     if (_cairo_operator_bounded_by_mask (op)) {
 	status = _cairo_scaled_font_glyph_device_extents (scaled_font,
-							  glyphs, 
-							  num_glyphs, 
+							  glyphs,
+							  num_glyphs,
 							  &glyph_extents);
 	if (status)
 	    return status;
 
 	_cairo_rectangle_intersect (&extents, &glyph_extents);
     }
-    
+
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
     if (status)
 	return status;
-    
+
     glyph_info.font = scaled_font;
     glyph_info.glyphs = glyphs;
     glyph_info.num_glyphs = num_glyphs;
-    
+
     status = _clip_and_composite (surface->clip,
 				  op,
 				  source,
@@ -949,7 +945,7 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 				  &glyph_info,
 				  surface,
 				  &extents);
-    
+
     return status;
 }
 
@@ -990,8 +986,8 @@ _cairo_surface_fallback_snapshot (cairo_surface_t *surface)
     _cairo_surface_release_source_image (surface,
 					 image, &image_extra);
 
-    snapshot->device_x_offset = surface->device_x_offset;
-    snapshot->device_y_offset = surface->device_y_offset;
+    snapshot->device_transform = surface->device_transform;
+    snapshot->device_transform_inverse = surface->device_transform_inverse;
 
     snapshot->is_snapshot = TRUE;
 
@@ -1042,11 +1038,11 @@ cairo_status_t
 _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
 					 cairo_operator_t	  op,
 					 const cairo_color_t	 *color,
-					 cairo_rectangle_fixed_t *rects,
+					 cairo_rectangle_int16_t *rects,
 					 int			  num_rects)
 {
     fallback_state_t state;
-    cairo_rectangle_fixed_t *offset_rects = NULL;
+    cairo_rectangle_int16_t *offset_rects = NULL;
     cairo_status_t status;
     int x1, y1, x2, y2;
     int i;
@@ -1063,7 +1059,7 @@ _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
     y1 = rects[0].y;
     x2 = rects[0].x + rects[0].width;
     y2 = rects[0].y + rects[0].height;
-    
+
     for (i = 1; i < num_rects; i++) {
 	if (rects[i].x < x1)
 	    x1 = rects[i].x;
@@ -1084,9 +1080,9 @@ _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
     }
 
     /* If the fetched image isn't at 0,0, we need to offset the rectangles */
-    
+
     if (state.image_rect.x != 0 || state.image_rect.y != 0) {
-	offset_rects = malloc (sizeof (cairo_rectangle_fixed_t) * num_rects);
+	offset_rects = malloc (sizeof (cairo_rectangle_int16_t) * num_rects);
 	if (offset_rects == NULL) {
 	    status = CAIRO_STATUS_NO_MEMORY;
 	    goto DONE;
@@ -1140,7 +1136,7 @@ _cairo_surface_fallback_composite_trapezoids (cairo_operator_t		op,
     }
 
     /* If the destination image isn't at 0,0, we need to offset the trapezoids */
-    
+
     if (state.image_rect.x != 0 || state.image_rect.y != 0) {
 	offset_traps = malloc (sizeof (cairo_trapezoid_t) * num_traps);
 	if (!offset_traps) {
@@ -1166,6 +1162,6 @@ _cairo_surface_fallback_composite_trapezoids (cairo_operator_t		op,
 
  DONE:
     _fallback_fini (&state);
-    
+
     return status;
 }
