@@ -103,7 +103,8 @@ typedef struct _cairo_type1_font_subset {
 
 static cairo_status_t
 _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
-				 cairo_type1_font_subset_t **subset_return)
+				 cairo_type1_font_subset_t **subset_return,
+                                 cairo_bool_t                hex_encode)
 {
     cairo_ft_unscaled_font_t *ft_unscaled_font;
     FT_Face face;
@@ -145,6 +146,7 @@ _cairo_type1_font_subset_create (cairo_unscaled_font_t      *unscaled_font,
     if (font->glyphs == NULL)
 	goto fail2;
 
+    font->hex_encode = hex_encode;
     font->num_glyphs = 0;
     for (i = 0; i < face->num_glyphs; i++)
 	font->glyphs[i].subset_index = -1;
@@ -185,7 +187,7 @@ static const unsigned short charstring_key = 4330;
 static cairo_bool_t
 is_ps_delimiter(int c)
 {
-    const static char delimiters[] = "()[]{}<>/% \t\r\n";
+    static const char delimiters[] = "()[]{}<>/% \t\r\n";
 
     return strchr (delimiters, c) != NULL;
 }
@@ -278,7 +280,7 @@ cairo_type1_font_subset_write_header (cairo_type1_font_subset_t *font,
     _cairo_output_stream_printf (font->output,
 				 "/Encoding 256 array\n"
 				 "0 1 255 {1 index exch /.notdef put} for\n");
-    for (i = 0; i < font->base.num_glyphs; i++) {
+    for (i = 1; i < font->base.num_glyphs; i++) {
 	if (font->glyphs[i].subset_index < 0)
 	    continue;
 	_cairo_output_stream_printf (font->output,
@@ -481,45 +483,86 @@ cairo_type1_font_subset_decode_integer (const unsigned char *p, int *integer)
 }
 
 static const char *ps_standard_encoding[256] = {
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/*   0 */
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/*  16 */
-    "space", "exclam", "quotedbl", "numbersign",	/*  32 */
-    "dollar", "percent", "ampersand", "quoteright",
-    "parenleft", "parenright", "asterisk", "plus",
-    "comma", "hyphen", "period", "slash",
-    "zero", "one", "two", "three",			/*  48 */
-    "four", "five", "six", "seven", "eight",
-    "nine", "colon", "semicolon", "less",
-    "equal", "greater", "question", "at",
-    "A", "B", "C", "D", "E", "F", "G", "H",		/*  64 */
-    "I", "J", "K", "L", "M", "N", "O", "P",
-    "Q", "R", "S", "T", "U", "V", "W", "X",		/*  80 */
-    "Y", "Z", "bracketleft", "backslash",
-    "bracketright", "asciicircum", "underscore", "quoteleft",
-    "a", "b", "c", "d", "e", "f", "g", "h",		/*  96 */
-    "i", "j", "k", "l", "m", "n", "o", "p",
-    "q", "r", "s", "t", "u", "v", "w", "x",		/* 112 */
-    "y", "z", "braceleft", "bar",
-    "braceright", "asciitilde", 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 128 */
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 144 */
-    "exclamdown", "cent", "sterling", "fraction",
-    "yen", "florin", "section", "currency",
-    "quotesingle", "quotedblleft", "guillemotleft", "guilsinglleft",
-    "guilsinglright", "fi", "fl", NULL,
-    "endash", "dagger", "daggerdbl", "periodcentered",	/* 160 */
-    NULL, "paragraph", "bullet", "quotesinglbase",
-    "quotedblbase", "quotedblright", "guillemotright", "ellipsis",
-    "perthousand", NULL, "questiondown", NULL,
-    "grave", "acute", "circumflex", "tilde",		/* 176 */
-    "macron", "breve", "dotaccent", "dieresis",
-    NULL, "ring", "cedilla", NULL,
-    "hungarumlaut", "ogonek", "caron", "emdash",
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 192 */
-    "AE", 0, "ordfeminine", 0, 0, 0, 0, "Lslash",	/* 208 */
-    "Oslash", "OE", "ordmasculine", 0, 0, 0, 0, 0,
-    "ae", 0, 0, 0, "dotlessi", 0, 0, "lslash",		/* 224 */
-    "oslash", "oe", "germandbls", 0, 0, 0, 0
+	/*   0 */
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	/*  16 */
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	/*  32 */
+	"space",	"exclam",	"quotedbl",	"numbersign",
+	"dollar",	"percent",	"ampersand",	"quoteright",
+	"parenleft",	"parenright",	"asterisk",	"plus",
+	"comma",	"hyphen",	"period",	"slash",
+	/*  48 */
+	"zero",		"one",		"two",		"three",
+	"four",		"five",		"six",		"seven",
+	"eight",	"nine",		"colon",	"semicolon",
+	"less",		"equal",	"greater",	"question",
+	/*  64 */
+	"at",		"A",		"B",		"C",
+	"D",		"E",		"F",		"G",
+	"H",		"I",		"J",		"K",
+	"L",		"M",		"N",		"O",
+	/*  80 */
+	"P",		"Q",		"R",		"S",
+	"T",		"U",		"V",		"W",
+	"X",		"Y",		"Z",		"bracketleft",
+	"backslash",	"bracketright",	"asciicircum",	"underscore",
+	/*  96 */
+	"quoteleft",	"a",		"b",		"c",
+	"d",		"e",		"f",		"g",
+	"h",		"i",		"j",		"k",
+	"l",		"m",		"n",		"o",
+	/* 112 */
+	"p",		"q",		"r",		"s",
+	"t",		"u",		"v",		"w",
+	"x",		"y",		"z",		"braceleft",
+	"bar",		"braceright",	"asciitilde",	NULL,
+	/* 128 */
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	/* 144 */
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	/* 160 */
+	NULL,		"exclamdown",	"cent",		"sterling",
+	"fraction",	"yen",		"florin",	"section",
+	"currency",	"quotesingle",	"quotedblleft",	"guillemotleft",
+	"guilsinglleft","guilsinglright","fi",		"fl",
+	/* 176 */
+	NULL,		"endash",	"dagger",	"daggerdbl",
+	"periodcentered",NULL,		"paragraph",	"bullet",
+	"quotesinglbase","quotedblbase","quotedblright","guillemotright",
+	"ellipsis",	"perthousand",	NULL,		"questiondown",
+	/* 192 */
+	NULL,		"grave",	"acute",	"circumflex",
+	"tilde",	"macron",	"breve",	"dotaccent",
+	"dieresis",	NULL,		"ring",		"cedilla",
+	NULL,		"hungarumlaut",	"ogonek",	"caron",
+	/* 208 */
+	"emdash",	NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	NULL,		NULL,		NULL,		NULL,
+	/* 224 */
+	NULL,		"AE",		NULL,		"ordfeminine",
+	NULL,		NULL,		NULL,		NULL,
+	"Lslash",	"Oslash",	"OE",		"ordmasculine",
+	NULL,		NULL,		NULL,		NULL,
+	/* 240 */
+	NULL,		"ae",		NULL,		NULL,
+	NULL,		"dotlessi",	NULL,		NULL,
+	"lslash",	"oslash",	"oe",		"germandbls",
+	NULL,		NULL,		NULL,		NULL
 };
 
 static void
@@ -859,7 +902,6 @@ cairo_type1_font_subset_write (cairo_type1_font_subset_t *font,
     }
 
     font->eexec_key = private_dict_key;
-    font->hex_encode = TRUE;
     font->hex_column = 0;
 
     cairo_type1_font_subset_write_private_dict (font, name);
@@ -953,7 +995,8 @@ cairo_type1_font_subset_destroy (void *abstract_font)
 cairo_status_t
 _cairo_type1_subset_init (cairo_type1_subset_t		*type1_subset,
 			  const char			*name,
-			  cairo_scaled_font_subset_t	*scaled_font_subset)
+			  cairo_scaled_font_subset_t	*scaled_font_subset,
+                          cairo_bool_t                   hex_encode)
 {
     cairo_type1_font_subset_t *font;
     cairo_status_t status;
@@ -967,7 +1010,7 @@ _cairo_type1_subset_init (cairo_type1_subset_t		*type1_subset,
 
     unscaled_font = _cairo_ft_scaled_font_get_unscaled_font (scaled_font_subset->scaled_font);
 
-    status = _cairo_type1_font_subset_create (unscaled_font, &font);
+    status = _cairo_type1_font_subset_create (unscaled_font, &font, hex_encode);
     if (status)
 	return status;
 
@@ -1038,4 +1081,3 @@ _cairo_type1_subset_fini (cairo_type1_subset_t *subset)
     free (subset->widths);
     free (subset->data);
 }
-
