@@ -37,14 +37,23 @@
 
 #include <stdio.h>
 #include "cairoint.h"
+#include "cairo-clip-private.h"
 #include "cairo-win32-private.h"
 
+/* for older SDKs */
+#ifndef SHADEBLENDCAPS
+#define SHADEBLENDCAPS  120
+#endif
+#ifndef SB_NONE
+#define SB_NONE         0x00000000
+#endif
+
 static const cairo_surface_backend_t cairo_win32_surface_backend;
 
 /**
  * _cairo_win32_print_gdi_error:
  * @context: context string to display along with the error
- * 
+ *
  * Helper function to dump out a human readable form of the
  * current error code.
  *
@@ -56,7 +65,7 @@ _cairo_win32_print_gdi_error (const char *context)
     void *lpMsgBuf;
     DWORD last_error = GetLastError ();
 
-    if (!FormatMessageA (FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+    if (!FormatMessageA (FORMAT_MESSAGE_ALLOCATE_BUFFER |
 			 FORMAT_MESSAGE_FROM_SYSTEM,
 			 NULL,
 			 last_error,
@@ -66,7 +75,7 @@ _cairo_win32_print_gdi_error (const char *context)
 	fprintf (stderr, "%s: Unknown GDI error", context);
     } else {
 	fprintf (stderr, "%s: %s", context, (char *)lpMsgBuf);
-	
+
 	LocalFree (lpMsgBuf);
     }
 
@@ -107,11 +116,11 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
     case CAIRO_FORMAT_RGB24:
 	num_palette = 0;
 	break;
-	
+
     case CAIRO_FORMAT_A8:
 	num_palette = 256;
 	break;
-	
+
     case CAIRO_FORMAT_A1:
 	num_palette = 2;
 	break;
@@ -132,16 +141,21 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
     bitmap_info->bmiHeader.biXPelsPerMeter = 72. / 0.0254; /* unused here */
     bitmap_info->bmiHeader.biYPelsPerMeter = 72. / 0.0254; /* unused here */
     bitmap_info->bmiHeader.biPlanes = 1;
-    
+
     switch (format) {
-    case CAIRO_FORMAT_ARGB32:
+    /* We can't create real RGB24 bitmaps because something seems to
+     * break if we do, especially if we don't set up an image
+     * fallback.  It could be a bug with using a 24bpp pixman image
+     * (and creating one with masks).  So treat them like 32bpp.
+     */
     case CAIRO_FORMAT_RGB24:
+    case CAIRO_FORMAT_ARGB32:
 	bitmap_info->bmiHeader.biBitCount = 32;
 	bitmap_info->bmiHeader.biCompression = BI_RGB;
 	bitmap_info->bmiHeader.biClrUsed = 0;	/* unused */
 	bitmap_info->bmiHeader.biClrImportant = 0;
 	break;
-	
+
     case CAIRO_FORMAT_A8:
 	bitmap_info->bmiHeader.biBitCount = 8;
 	bitmap_info->bmiHeader.biCompression = BI_RGB;
@@ -154,9 +168,9 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
 	    bitmap_info->bmiColors[i].rgbRed = i;
 	    bitmap_info->bmiColors[i].rgbReserved = 0;
 	}
-	
+
 	break;
-	
+
     case CAIRO_FORMAT_A1:
 	bitmap_info->bmiHeader.biBitCount = 1;
 	bitmap_info->bmiHeader.biCompression = BI_RGB;
@@ -184,11 +198,13 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
     if (!surface->bitmap)
 	goto FAIL;
 
+    GdiFlush();
+
     surface->saved_dc_bitmap = SelectObject (surface->dc,
 					     surface->bitmap);
     if (!surface->saved_dc_bitmap)
 	goto FAIL;
-    
+
     if (bitmap_info && num_palette > 2)
 	free (bitmap_info);
 
@@ -202,11 +218,11 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
 	case CAIRO_FORMAT_RGB24:
 	    *rowstride_out = 4 * width;
 	    break;
-	    
+
 	case CAIRO_FORMAT_A8:
 	    *rowstride_out = (width + 3) & ~3;
 	    break;
-	
+
 	case CAIRO_FORMAT_A1:
 	    *rowstride_out = ((width + 31) & ~31) / 8;
 	    break;
@@ -217,7 +233,7 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
 
  FAIL:
     status = _cairo_win32_print_gdi_error ("_create_dc_and_bitmap");
-    
+
     if (bitmap_info && num_palette > 2)
 	free (bitmap_info);
 
@@ -225,17 +241,17 @@ _create_dc_and_bitmap (cairo_win32_surface_t *surface,
 	SelectObject (surface->dc, surface->saved_dc_bitmap);
 	surface->saved_dc_bitmap = NULL;
     }
-    
+
     if (surface->bitmap) {
 	DeleteObject (surface->bitmap);
 	surface->bitmap = NULL;
     }
-    
+
     if (surface->dc) {
  	DeleteDC (surface->dc);
 	surface->dc = NULL;
     }
- 
+
     return status;
 }
 
@@ -250,6 +266,8 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
     char *bits;
     int rowstride;
 
+    _cairo_win32_initialize ();
+
     surface = malloc (sizeof (cairo_win32_surface_t));
     if (surface == NULL) {
 	_cairo_error (CAIRO_STATUS_NO_MEMORY);
@@ -268,9 +286,9 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
 	status = CAIRO_STATUS_NO_MEMORY;
 	goto FAIL;
     }
-    
+
     surface->format = format;
-    
+
     surface->clip_rect.x = 0;
     surface->clip_rect.y = 0;
     surface->clip_rect.width = width;
@@ -284,7 +302,8 @@ _cairo_win32_surface_create_for_dc (HDC             original_dc,
 
     surface->extents = surface->clip_rect;
 
-    _cairo_surface_init (&surface->base, &cairo_win32_surface_backend);
+    _cairo_surface_init (&surface->base, &cairo_win32_surface_backend,
+			 _cairo_content_from_format (format));
 
     return (cairo_surface_t *)surface;
 
@@ -351,15 +370,15 @@ _cairo_win32_surface_get_subimage (cairo_win32_surface_t  *surface,
     cairo_status_t status;
     cairo_content_t content = _cairo_content_from_format (surface->format);
 
-    local = 
+    local =
 	(cairo_win32_surface_t *) _cairo_win32_surface_create_similar (surface,
 								       content,
 								       width,
 								       height);
     if (local->base.status)
 	return CAIRO_STATUS_NO_MEMORY;
-    
-    if (!BitBlt (local->dc, 
+
+    if (!BitBlt (local->dc,
 		 0, 0,
 		 width, height,
 		 surface->dc,
@@ -378,7 +397,7 @@ _cairo_win32_surface_get_subimage (cairo_win32_surface_t  *surface,
     }
 
     *local_out = local;
-    
+
     return CAIRO_STATUS_SUCCESS;
 
  FAIL:
@@ -424,16 +443,16 @@ _cairo_win32_surface_release_source_image (void                   *abstract_surf
 					   void                   *image_extra)
 {
     cairo_win32_surface_t *local = image_extra;
-    
+
     if (local)
 	cairo_surface_destroy ((cairo_surface_t *)local);
 }
 
 static cairo_status_t
 _cairo_win32_surface_acquire_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_fixed_t *interest_rect,
+					 cairo_rectangle_int16_t *interest_rect,
 					 cairo_image_surface_t  **image_out,
-					 cairo_rectangle_fixed_t *image_rect,
+					 cairo_rectangle_int16_t *image_rect,
 					 void                   **image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -443,6 +462,8 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
     int x1, y1, x2, y2;
 
     if (surface->image) {
+	GdiFlush();
+
 	image_rect->x = 0;
 	image_rect->y = 0;
 	image_rect->width = surface->clip_rect.width;
@@ -470,15 +491,15 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
 	x2 = interest_rect->x + interest_rect->width;
     if (interest_rect->y + interest_rect->height < y2)
 	y2 = interest_rect->y + interest_rect->height;
-    
+
     if (x1 >= x2 || y1 >= y2) {
 	*image_out = NULL;
 	*image_extra = NULL;
-	
+
 	return CAIRO_STATUS_SUCCESS;
     }
-	
-    status = _cairo_win32_surface_get_subimage (abstract_surface, 
+
+    status = _cairo_win32_surface_get_subimage (abstract_surface,
 						x1, y1, x2 - x1, y2 - y1,
 						&local);
     if (status)
@@ -486,7 +507,7 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
 
     *image_out = (cairo_image_surface_t *)local->image;
     *image_extra = local;
-    
+
     image_rect->x = x1;
     image_rect->y = y1;
     image_rect->width = x2 - x1;
@@ -497,14 +518,14 @@ _cairo_win32_surface_acquire_dest_image (void                    *abstract_surfa
 
 static void
 _cairo_win32_surface_release_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_fixed_t *interest_rect,
+					 cairo_rectangle_int16_t *interest_rect,
 					 cairo_image_surface_t   *image,
-					 cairo_rectangle_fixed_t *image_rect,
+					 cairo_rectangle_int16_t *image_rect,
 					 void                    *image_extra)
 {
     cairo_win32_surface_t *surface = abstract_surface;
     cairo_win32_surface_t *local = image_extra;
-    
+
     if (!local)
 	return;
 
@@ -569,22 +590,22 @@ _composite_alpha_blend (cairo_win32_surface_t *dst,
      */
     if (!alpha_blend_checked) {
 	OSVERSIONINFO os;
-    
+
 	os.dwOSVersionInfoSize = sizeof (os);
 	GetVersionEx (&os);
-	
+
 	/* If running on Win98, disable using AlphaBlend()
 	 * to avoid Win98 AlphaBlend() bug */
 	if (VER_PLATFORM_WIN32_WINDOWS != os.dwPlatformId ||
 	    os.dwMajorVersion != 4 || os.dwMinorVersion != 10)
 	{
 	    HMODULE msimg32_dll = LoadLibrary ("msimg32");
-	    
+
 	    if (msimg32_dll != NULL)
 		alpha_blend = (cairo_alpha_blend_func_t)GetProcAddress (msimg32_dll,
 									"AlphaBlend");
 	}
-	    
+
 	alpha_blend_checked = TRUE;
     }
 
@@ -592,7 +613,7 @@ _composite_alpha_blend (cairo_win32_surface_t *dst,
 	return CAIRO_INT_STATUS_UNSUPPORTED;
     if (GetDeviceCaps(dst->dc, SHADEBLENDCAPS) == SB_NONE)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
-    
+
     blend_function.BlendOp = AC_SRC_OVER;
     blend_function.BlendFlags = 0;
     blend_function.SourceConstantAlpha = alpha;
@@ -606,7 +627,7 @@ _composite_alpha_blend (cairo_win32_surface_t *dst,
 		      width, height,
 		      blend_function))
 	return _cairo_win32_print_gdi_error ("_cairo_win32_surface_composite");
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -652,7 +673,7 @@ _cairo_win32_surface_composite (cairo_operator_t	op,
 
     if (src->base.backend != dst->base.backend)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
-    
+
     integer_transform = _cairo_matrix_is_integer_translation (&pattern->matrix, &itx, &ity);
     if (!integer_transform)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
@@ -685,10 +706,9 @@ _cairo_win32_surface_composite (cairo_operator_t	op,
         height = src->extents.height - src_y;
 
     if (alpha == 255 &&
-	src->format == dst->format &&
 	(op == CAIRO_OPERATOR_SOURCE ||
 	 (src->format == CAIRO_FORMAT_RGB24 && op == CAIRO_OPERATOR_OVER))) {
-	
+
 	if (!BitBlt (dst->dc,
 		     dst_x, dst_y,
 		     width, height,
@@ -698,17 +718,16 @@ _cairo_win32_surface_composite (cairo_operator_t	op,
 	    return _cairo_win32_print_gdi_error ("_cairo_win32_surface_composite");
 
 	return CAIRO_STATUS_SUCCESS;
-	
-    } else if (integer_transform &&
-	       (src->format == CAIRO_FORMAT_RGB24 || src->format == CAIRO_FORMAT_ARGB32) &&
-	       dst->format == CAIRO_FORMAT_RGB24 &&
+
+    } else if ((src->format == CAIRO_FORMAT_RGB24 || src->format == CAIRO_FORMAT_ARGB32) &&
+	       (dst->format == CAIRO_FORMAT_RGB24 || dst->format == CAIRO_FORMAT_ARGB32) &&
 	       op == CAIRO_OPERATOR_OVER) {
 
 	return _composite_alpha_blend (dst, src, alpha,
 				       src_x, src_y,
 				       dst_x, dst_y, width, height);
     }
-    
+
     return CAIRO_INT_STATUS_UNSUPPORTED;
 }
 
@@ -730,13 +749,13 @@ categorize_solid_dest_operator (cairo_operator_t op,
 	source = SOURCE_TRANSPARENT;
     else
 	source = SOURCE_OTHER;
-    
+
     switch (op) {
     case CAIRO_OPERATOR_CLEAR:    /* 0                 0 */
     case CAIRO_OPERATOR_OUT:      /* 1 - Ab            0 */
 	return DO_CLEAR;
 	break;
-	
+
     case CAIRO_OPERATOR_SOURCE:   /* 1                 0 */
     case CAIRO_OPERATOR_IN:       /* Ab                0 */
 	return DO_SOURCE;
@@ -751,7 +770,7 @@ categorize_solid_dest_operator (cairo_operator_t op,
 	else
 	    return DO_UNSUPPORTED;
 	break;
-	
+
     case CAIRO_OPERATOR_DEST_OUT: /* 0            1 - Aa */
     case CAIRO_OPERATOR_XOR:      /* 1 - Ab       1 - Aa */
 	if (source == SOURCE_SOLID)
@@ -761,7 +780,7 @@ categorize_solid_dest_operator (cairo_operator_t op,
 	else
 	    return DO_UNSUPPORTED;
     	break;
-	
+
     case CAIRO_OPERATOR_DEST:     /* 0                 1 */
     case CAIRO_OPERATOR_DEST_OVER:/* 1 - Ab            1 */
     case CAIRO_OPERATOR_SATURATE: /* min(1,(1-Ab)/Aa)  1 */
@@ -777,14 +796,14 @@ categorize_solid_dest_operator (cairo_operator_t op,
 	else
 	    return DO_UNSUPPORTED;
 	break;
-	
+
     case CAIRO_OPERATOR_ADD:	  /* 1                1 */
 	if (source == SOURCE_TRANSPARENT)
 	    return DO_NOTHING;
 	else
 	    return DO_UNSUPPORTED;
 	break;
-    }	
+    }
 
     ASSERT_NOT_REACHED;
     return DO_UNSUPPORTED;
@@ -794,7 +813,7 @@ static cairo_int_status_t
 _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
 				      cairo_operator_t		op,
 				      const cairo_color_t	*color,
-				      cairo_rectangle_fixed_t		*rects,
+				      cairo_rectangle_int16_t		*rects,
 				      int			num_rects)
 {
     cairo_win32_surface_t *surface = abstract_surface;
@@ -803,10 +822,11 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
     HBRUSH new_brush;
     int i;
 
-    /* If we have a local image, use the fallback code; it will be as fast
-     * as calling out to GDI.
-     */
-    if (surface->image)
+    /* XXXperf If it's not RGB24, we need to do a little more checking
+     * to figure out when we can use GDI.  We don't have that checking
+     * anywhere at the moment, so just bail and use the fallback
+     * paths. */
+    if (surface->format != CAIRO_FORMAT_RGB24)
 	return CAIRO_INT_STATUS_UNSUPPORTED;
 
     /* Optimize for no destination alpha (surface->pixman_image is non-NULL for all
@@ -815,7 +835,7 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
     switch (categorize_solid_dest_operator (op, color->alpha_short)) {
     case DO_CLEAR:
 	new_color = RGB (0, 0, 0);
-	break;	
+	break;
     case DO_SOURCE:
 	new_color = RGB (color->red_short >> 8, color->green_short >> 8, color->blue_short >> 8);
 	break;
@@ -825,11 +845,11 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
     default:
 	return CAIRO_INT_STATUS_UNSUPPORTED;
     }
-    
+
     new_brush = CreateSolidBrush (new_color);
     if (!new_brush)
 	return _cairo_win32_print_gdi_error ("_cairo_win32_surface_fill_rectangles");
-    
+
     for (i = 0; i < num_rects; i++) {
 	RECT rect;
 
@@ -843,14 +863,14 @@ _cairo_win32_surface_fill_rectangles (void			*abstract_surface,
     }
 
     DeleteObject (new_brush);
-    
+
     return CAIRO_STATUS_SUCCESS;
 
  FAIL:
     status = _cairo_win32_print_gdi_error ("_cairo_win32_surface_fill_rectangles");
-    
+
     DeleteObject (new_brush);
-    
+
     return status;
 }
 
@@ -883,7 +903,7 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 	    return _cairo_win32_print_gdi_error ("_cairo_win32_surface_set_clip_region (reset)");
 
 	return CAIRO_STATUS_SUCCESS;
-    
+
     } else {
 	pixman_box16_t *boxes = pixman_region_rects (region);
 	int num_boxes = pixman_region_num_rects (region);
@@ -920,7 +940,7 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 
 	gdi_region = ExtCreateRegion (NULL, data_size, data);
 	free (data);
-	
+
 	if (!gdi_region)
 	    return CAIRO_STATUS_NO_MEMORY;
 
@@ -946,7 +966,7 @@ _cairo_win32_surface_set_clip_region (void              *abstract_surface,
 
 static cairo_int_status_t
 _cairo_win32_surface_get_extents (void		          *abstract_surface,
-				  cairo_rectangle_fixed_t *rectangle)
+				  cairo_rectangle_int16_t *rectangle)
 {
     cairo_win32_surface_t *surface = abstract_surface;
 
@@ -964,13 +984,13 @@ _cairo_win32_surface_flush (void *abstract_surface)
 /**
  * cairo_win32_surface_create:
  * @hdc: the DC to create a surface for
- * 
+ *
  * Creates a cairo surface that targets the given DC.  The DC will be
  * queried for its initial clip extents, and this will be used as the
  * size of the cairo surface.  Also, if the DC is a raster DC, it will
  * be queried for its pixel format and the cairo surface format will
  * be set appropriately.
- * 
+ *
  * Return value: the newly created surface
  **/
 cairo_surface_t *
@@ -981,6 +1001,8 @@ cairo_win32_surface_create (HDC hdc)
     int depth;
     cairo_format_t format;
 
+    _cairo_win32_initialize ();
+
     /* Try to figure out the drawing bounds for the Device context
      */
     if (GetClipBox (hdc, &rect) == ERROR) {
@@ -993,11 +1015,11 @@ cairo_win32_surface_create (HDC hdc)
     if (GetDeviceCaps(hdc, TECHNOLOGY) == DT_RASDISPLAY) {
 	depth = GetDeviceCaps(hdc, BITSPIXEL);
 	if (depth == 32)
-	    format = CAIRO_FORMAT_ARGB32;
+	    format = CAIRO_FORMAT_RGB24;
 	else if (depth == 24)
 	    format = CAIRO_FORMAT_RGB24;
-        else if (depth == 16)
-            format = CAIRO_FORMAT_RGB24;
+	else if (depth == 16)
+	    format = CAIRO_FORMAT_RGB24;
 	else if (depth == 8)
 	    format = CAIRO_FORMAT_A8;
 	else if (depth == 1)
@@ -1019,11 +1041,11 @@ cairo_win32_surface_create (HDC hdc)
 
     surface->image = NULL;
     surface->format = format;
-    
+
     surface->dc = hdc;
     surface->bitmap = NULL;
     surface->saved_dc_bitmap = NULL;
-    
+
     surface->clip_rect.x = rect.left;
     surface->clip_rect.y = rect.top;
     surface->clip_rect.width = rect.right - rect.left;
@@ -1043,23 +1065,25 @@ cairo_win32_surface_create (HDC hdc)
 
     surface->extents = surface->clip_rect;
 
-    _cairo_surface_init (&surface->base, &cairo_win32_surface_backend);
+    _cairo_surface_init (&surface->base, &cairo_win32_surface_backend,
+			 _cairo_content_from_format (format));
 
     return (cairo_surface_t *)surface;
 }
 
 /**
  * cairo_win32_surface_create_with_dib:
- * @format: format of pixels in the surface to create 
+ * @format: format of pixels in the surface to create
  * @width: width of the surface, in pixels
  * @height: height of the surface, in pixels
- * 
+ *
  * Creates a device-independent-bitmap surface not associated with
  * any particular existing surface or device context. The created
  * bitmap will be unititialized.
- * 
+ *
  * Return value: the newly created surface
  *
+ * Since: 1.2
  **/
 cairo_surface_t *
 cairo_win32_surface_create_with_dib (cairo_format_t format,
@@ -1069,13 +1093,12 @@ cairo_win32_surface_create_with_dib (cairo_format_t format,
     return _cairo_win32_surface_create_for_dc (NULL, format, width, height);
 }
 
-
 /**
  * _cairo_surface_is_win32:
  * @surface: a #cairo_surface_t
- * 
+ *
  * Checks if a surface is an #cairo_win32_surface_t
- * 
+ *
  * Return value: True if the surface is an win32 surface
  **/
 int
@@ -1092,6 +1115,8 @@ _cairo_surface_is_win32 (cairo_surface_t *surface)
  * Also returns NULL if the surface is not a win32 surface.
  *
  * Return value: HDC or NULL if no HDC available.
+ *
+ * Since: 1.2
  **/
 HDC
 cairo_win32_surface_get_dc (cairo_surface_t *surface)
@@ -1129,7 +1154,17 @@ static const cairo_surface_backend_t cairo_win32_surface_backend = {
     NULL, /* old_show_glyphs */
     NULL, /* get_font_options */
     _cairo_win32_surface_flush,
-    NULL  /* mark_dirty_rectangle */
+    NULL, /* mark_dirty_rectangle */
+    NULL, /* scaled_font_fini */
+    NULL, /* scaled_glyph_fini */
+
+    NULL, /* paint */
+    NULL, /* mask */
+    NULL, /* stroke */
+    NULL, /* fill */
+    NULL, /* show_glyphs */
+
+    NULL  /* snapshot */
 };
 
 /*
@@ -1138,11 +1173,28 @@ static const cairo_surface_backend_t cairo_win32_surface_backend = {
  * threaded before any other function.
  * Initializing more than finally needed should not matter much.
  */
-#ifndef HAVE_PTHREAD_H
+#if !defined(HAVE_PTHREAD_H) 
+
 CRITICAL_SECTION cairo_toy_font_face_hash_table_mutex;
 CRITICAL_SECTION cairo_scaled_font_map_mutex;
 CRITICAL_SECTION cairo_ft_unscaled_font_map_mutex;
 
+static int _cairo_win32_initialized = 0;
+
+void
+_cairo_win32_initialize () {
+    if (_cairo_win32_initialized)
+        return;
+
+    /* every 'mutex' from CAIRO_MUTEX_DECALRE needs to be initialized here */
+    InitializeCriticalSection (&cairo_toy_font_face_hash_table_mutex);
+    InitializeCriticalSection (&cairo_scaled_font_map_mutex);
+    InitializeCriticalSection (&cairo_ft_unscaled_font_map_mutex);
+
+    _cairo_win32_initialized = 1;
+}
+
+#if !defined(CAIRO_WIN32_STATIC_BUILD)
 BOOL WINAPI
 DllMain (HINSTANCE hinstDLL,
 	 DWORD     fdwReason,
@@ -1151,10 +1203,7 @@ DllMain (HINSTANCE hinstDLL,
   switch (fdwReason)
   {
   case DLL_PROCESS_ATTACH:
-    /* every 'mutex' from CAIRO_MUTEX_DECALRE needs to be initialized here */
-    InitializeCriticalSection (&cairo_toy_font_face_hash_table_mutex);
-    InitializeCriticalSection (&cairo_scaled_font_map_mutex);
-    InitializeCriticalSection (&cairo_ft_unscaled_font_map_mutex);
+    _cairo_win32_initialize();
     break;
   case DLL_PROCESS_DETACH:
     DeleteCriticalSection (&cairo_toy_font_face_hash_table_mutex);
@@ -1165,3 +1214,4 @@ DllMain (HINSTANCE hinstDLL,
   return TRUE;
 }
 #endif
+#endif
