@@ -1,5 +1,5 @@
 /*
- * Copyright © 2004 Red Hat, Inc.
+ * Copyright © 2004,2006 Red Hat, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software
  * and its documentation for any purpose is hereby granted without
@@ -32,7 +32,9 @@
 #include <stdarg.h>
 #include <ctype.h>
 #include <setjmp.h>
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif
 #include <assert.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -46,8 +48,6 @@
 #include "cairo-test.h"
 
 #include "buffer-diff.h"
-#include "read-png.h"
-#include "write-png.h"
 #include "xmalloc.h"
 
 /* This is copied from cairoint.h. That makes it painful to keep in
@@ -117,6 +117,29 @@ const char *srcdir;
  * an nonrecoverable fashion. */
 jmp_buf jmpbuf;
 
+typedef cairo_surface_t *
+(*cairo_test_create_target_surface_t) (cairo_test_t	 *test,
+				       cairo_content_t	  content,
+				       void		**closure);
+
+typedef cairo_status_t
+(*cairo_test_write_to_png_t) (cairo_surface_t *surface, const char *filename);
+
+typedef void
+(*cairo_test_cleanup_target_t) (void *closure);
+
+typedef struct _cairo_test_target
+{
+    const char		       	       *name;
+    cairo_surface_type_t		expected_type;
+    cairo_content_t			content;
+    int					error_tolerance;
+    cairo_test_create_target_surface_t	create_target_surface;
+    cairo_test_write_to_png_t		write_to_png;
+    cairo_test_cleanup_target_t		cleanup_target;
+    void			       *closure;
+} cairo_test_target_t;
+
 void
 cairo_test_init (const char *test_name)
 {
@@ -199,28 +222,6 @@ xunlink (const char *pathname)
     }
 }
 
-typedef cairo_surface_t *
-(*cairo_test_create_target_surface_t) (cairo_test_t	 *test,
-				       cairo_content_t	  content,
-				       void		**closure);
-
-typedef cairo_status_t
-(*cairo_test_write_to_png_t) (cairo_surface_t *surface, const char *filename);
-
-typedef void
-(*cairo_test_cleanup_target_t) (void *closure);
-
-typedef struct _cairo_test_target
-{
-    const char		       	       *name;
-    cairo_surface_type_t		expected_type;
-    cairo_content_t			content;
-    cairo_test_create_target_surface_t	create_target_surface;
-    cairo_test_write_to_png_t		write_to_png;
-    cairo_test_cleanup_target_t		cleanup_target;
-    void			       *closure;
-} cairo_test_target_t;
-
 static const char *
 _cairo_test_content_name (cairo_content_t content)
 {
@@ -1525,12 +1526,7 @@ cairo_ref_name_for_test_target_format (const char *test_name,
     else
 	goto done;
 
-    xasprintf (&ref_name, "/dev/null");
-    cairo_test_log ("Error: Cannot find reference image for %s/%s-%s-%s%s\n",srcdir,
-		    test_name,
-		    target_name,
-		    format,
-		    CAIRO_TEST_REF_SUFFIX);
+    ref_name = NULL;
 
 done:
     return ref_name;
@@ -1651,20 +1647,42 @@ cairo_test_for_target (cairo_test_t		 *test,
 
     /* Skip image check for tests with no image (width,height == 0,0) */
     if (test->width != 0 && test->height != 0) {
-	int pixels_changed;
+	buffer_diff_result_t result;
+	cairo_status_t diff_status;
 	xunlink (png_name);
 	(target->write_to_png) (surface, png_name);
-	if (target->content == CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED)
-	    pixels_changed = image_diff_flattened (png_name, ref_name, diff_name, dev_offset, dev_offset, 0, 0);
-	else
-	    pixels_changed = image_diff (png_name, ref_name, diff_name, dev_offset, dev_offset, 0, 0);
-	if (pixels_changed) {
-	    if (pixels_changed > 0)
-		cairo_test_log ("Error: %d pixels differ from reference image %s\n",
-				pixels_changed, ref_name);
+
+	if (!ref_name) {
+	    cairo_test_log ("Error: Cannot find reference image for %s/%s-%s-%s%s\n",srcdir,
+			    test->name,
+			    target->name,
+			    format,
+			    CAIRO_TEST_REF_SUFFIX);
 	    ret = CAIRO_TEST_FAILURE;
 	    goto UNWIND_CAIRO;
 	}
+
+	if (target->content == CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED) {
+	    diff_status= image_diff_flattened (png_name, ref_name, diff_name,
+					       dev_offset, dev_offset, 0, 0, &result);
+	} else {
+	    diff_status = image_diff (png_name, ref_name, diff_name,
+				      dev_offset, dev_offset, 0, 0, &result);
+	}
+	if (diff_status) {
+	    cairo_test_log ("Error: Failed to compare images: %s\n",
+			    cairo_status_to_string (diff_status));
+	    ret = CAIRO_TEST_FAILURE;
+	    goto UNWIND_CAIRO;
+	}
+	if (result.pixels_changed) {
+	    cairo_test_log ("%d pixels differ (with maximum difference of %d) from reference image %s\n",
+			    result.pixels_changed, result.max_diff, ref_name);
+	    if (result.max_diff > target->error_tolerance) {
+		ret = CAIRO_TEST_FAILURE;
+		goto UNWIND_CAIRO;
+	    }
+	}
     }
 
     ret = CAIRO_TEST_SUCCESS;
@@ -1680,19 +1698,25 @@ UNWIND_SURFACE:
 	target->cleanup_target (target->closure);
 
 UNWIND_STRINGS:
-    free (png_name);
-    free (ref_name);
-    free (diff_name);
-    free (offset_str);
+    if (png_name)
+      free (png_name);
+    if (ref_name)
+      free (ref_name);
+    if (diff_name)
+      free (diff_name);
+    if (offset_str)
+      free (offset_str);
 
     return ret;
 }
 
+#ifdef HAVE_SIGNAL_H
 static void
 segfault_handler (int signal)
 {
     longjmp (jmpbuf, signal);
 }
+#endif
 
 static cairo_test_status_t
 cairo_test_expecting (cairo_test_t *test,
@@ -1703,87 +1727,96 @@ cairo_test_expecting (cairo_test_t *test,
     volatile size_t i, j, num_targets;
     volatile cairo_bool_t limited_targets = FALSE, print_fail_on_stdout = TRUE;
     const char *tname;
+#ifdef HAVE_SIGNAL_H
     void (*old_segfault_handler)(int);
+#endif
     volatile cairo_test_status_t status, ret;
     cairo_test_target_t ** volatile targets_to_test;
     cairo_test_target_t targets[] =
 	{
-	    { "image", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR_ALPHA,
+	    /* I'm uncompromising about leaving the image backend as 0
+	     * for tolerance. There shouldn't ever be anything that is out of
+	     * our control here. */
+	    { "image", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR_ALPHA, 0,
 	      create_image_surface, cairo_surface_write_to_png, NULL},
-	    { "image", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR,
+	    { "image", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR, 0,
 	      create_image_surface, cairo_surface_write_to_png, NULL},
 #ifdef CAIRO_HAS_TEST_SURFACES
 	    { "test-fallback", CAIRO_INTERNAL_SURFACE_TYPE_TEST_FALLBACK,
-	      CAIRO_CONTENT_COLOR_ALPHA,
+	      CAIRO_CONTENT_COLOR_ALPHA, 0,
 	      create_test_fallback_surface, cairo_surface_write_to_png, NULL },
 	    { "test-fallback", CAIRO_INTERNAL_SURFACE_TYPE_TEST_FALLBACK,
-	      CAIRO_CONTENT_COLOR,
+	      CAIRO_CONTENT_COLOR, 0,
 	      create_test_fallback_surface, cairo_surface_write_to_png, NULL },
 	    { "test-meta", CAIRO_INTERNAL_SURFACE_TYPE_TEST_META,
-	      CAIRO_CONTENT_COLOR_ALPHA,
+	      CAIRO_CONTENT_COLOR_ALPHA, 0,
 	      create_test_meta_surface, cairo_surface_write_to_png, NULL },
 	    { "test-meta", CAIRO_INTERNAL_SURFACE_TYPE_TEST_META,
-	      CAIRO_CONTENT_COLOR,
+	      CAIRO_CONTENT_COLOR, 0,
 	      create_test_meta_surface, cairo_surface_write_to_png, NULL },
 	    { "test-paginated", CAIRO_INTERNAL_SURFACE_TYPE_TEST_PAGINATED,
-	      CAIRO_CONTENT_COLOR_ALPHA,
+	      CAIRO_CONTENT_COLOR_ALPHA, 0,
 	      create_test_paginated_surface,
 	      test_paginated_write_to_png,
 	      cleanup_test_paginated },
 	    { "test-paginated", CAIRO_INTERNAL_SURFACE_TYPE_TEST_PAGINATED,
-	      CAIRO_CONTENT_COLOR,
+	      CAIRO_CONTENT_COLOR, 0,
 	      create_test_paginated_surface,
 	      test_paginated_write_to_png,
 	      cleanup_test_paginated },
 #endif
 #ifdef CAIRO_HAS_GLITZ_SURFACE
 #if CAIRO_CAN_TEST_GLITZ_GLX_SURFACE
-	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ,CAIRO_CONTENT_COLOR_ALPHA,
+	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ,CAIRO_CONTENT_COLOR_ALPHA, 0,
 		create_cairo_glitz_glx_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_glx },
-	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
+	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR, 0,
 		create_cairo_glitz_glx_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_glx },
 #endif
 #if CAIRO_CAN_TEST_GLITZ_AGL_SURFACE
-	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA,
+	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA, 0,
 		create_cairo_glitz_agl_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_agl },
-	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
+	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR, 0,
 		create_cairo_glitz_agl_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_agl },
 #endif
 #if CAIRO_CAN_TEST_GLITZ_WGL_SURFACE
-	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA,
+	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA, 0,
 		create_cairo_glitz_wgl_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_wgl },
-	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
+	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR, 0,
 		create_cairo_glitz_wgl_surface, cairo_surface_write_to_png,
 		cleanup_cairo_glitz_wgl },
 #endif
 #endif /* CAIRO_HAS_GLITZ_SURFACE */
 #if 0 && CAIRO_HAS_QUARTZ_SURFACE
-	    { "quartz", CAIRO_SURFACE_TYPE_QUARTZ, CAIRO_CONTENT_COLOR,
+	    { "quartz", CAIRO_SURFACE_TYPE_QUARTZ, CAIRO_CONTENT_COLOR, 0,
 		create_quartz_surface, cairo_surface_write_to_png,
 		cleanup_quartz },
 #endif
 #if CAIRO_HAS_WIN32_SURFACE
-	    { "win32", CAIRO_SURFACE_TYPE_WIN32, CAIRO_CONTENT_COLOR,
+	    { "win32", CAIRO_SURFACE_TYPE_WIN32, CAIRO_CONTENT_COLOR, 0,
 		create_win32_surface, cairo_surface_write_to_png, cleanup_win32 },
 #endif
 #if CAIRO_HAS_XCB_SURFACE
-	    { "xcb", CAIRO_SURFACE_TYPE_XCB, CAIRO_CONTENT_COLOR_ALPHA,
+	    /* Acceleration architectures may make the results differ by a
+	     * bit, so we set the error tolerance to 1. */
+	    { "xcb", CAIRO_SURFACE_TYPE_XCB, CAIRO_CONTENT_COLOR_ALPHA, 1,
 		create_xcb_surface, cairo_surface_write_to_png, cleanup_xcb},
 #endif
 #if CAIRO_HAS_XLIB_SURFACE
-	    { "xlib", CAIRO_SURFACE_TYPE_XLIB, CAIRO_CONTENT_COLOR_ALPHA,
+	    /* Acceleration architectures may make the results differ by a
+	     * bit, so we set the error tolerance to 1. */
+	    { "xlib", CAIRO_SURFACE_TYPE_XLIB, CAIRO_CONTENT_COLOR_ALPHA, 1,
 		create_xlib_surface, cairo_surface_write_to_png, cleanup_xlib},
-	    { "xlib", CAIRO_SURFACE_TYPE_XLIB, CAIRO_CONTENT_COLOR,
+	    { "xlib", CAIRO_SURFACE_TYPE_XLIB, CAIRO_CONTENT_COLOR, 1,
 		create_xlib_surface, cairo_surface_write_to_png, cleanup_xlib},
 #endif
 #if CAIRO_HAS_PS_SURFACE
 	    { "ps", CAIRO_SURFACE_TYPE_PS,
-	        CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED,
+	        CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED, 0,
 		create_ps_surface, ps_surface_write_to_png, cleanup_ps },
 
 	    /* XXX: We expect type image here only due to a limitation in
@@ -1794,12 +1827,12 @@ cairo_test_expecting (cairo_test_t *test,
 	     * have source support for PS/meta-surfaces, so the
 	     * create_similar path for all paginated surfaces currently
 	     * returns an image surface.*/
-	    { "ps", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR,
+	    { "ps", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR, 0,
 		create_ps_surface, ps_surface_write_to_png, cleanup_ps },
 #endif
 #if CAIRO_HAS_PDF_SURFACE && CAIRO_CAN_TEST_PDF_SURFACE
 	    { "pdf", CAIRO_SURFACE_TYPE_PDF,
-	        CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED,
+	        CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED, 0,
 		create_pdf_surface, pdf_surface_write_to_png, cleanup_pdf },
 
 	    /* XXX: We expect type image here only due to a limitation in
@@ -1810,28 +1843,33 @@ cairo_test_expecting (cairo_test_t *test,
 	     * have source support for PDF/meta-surfaces, so the
 	     * create_similar path for all paginated surfaces currently
 	     * returns an image surface.*/
-	    { "pdf", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR,
+	    { "pdf", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR, 0,
 		create_pdf_surface, pdf_surface_write_to_png, cleanup_pdf },
 #endif
 #if CAIRO_HAS_SVG_SURFACE && CAIRO_CAN_TEST_SVG_SURFACE
-	    { "svg", CAIRO_SURFACE_TYPE_SVG, CAIRO_CONTENT_COLOR_ALPHA,
+	    /* It seems we should be able to round-trip SVG content perfrectly
+	     * through librsvg and cairo, but for some mysterious reason, some
+	     * systems get an error of 1 for some pixels on some of the text
+	     * tests. XXX: I'd still like to chase these down at some point.
+	     * For now just set the svg error tolerance to 1. */
+	    { "svg", CAIRO_SURFACE_TYPE_SVG, CAIRO_CONTENT_COLOR_ALPHA, 1,
 		    create_svg_surface, svg_surface_write_to_png, cleanup_svg },
-	    { "svg", CAIRO_INTERNAL_SURFACE_TYPE_META, CAIRO_CONTENT_COLOR,
+	    { "svg", CAIRO_INTERNAL_SURFACE_TYPE_META, CAIRO_CONTENT_COLOR, 1,
 		    create_svg_surface, svg_surface_write_to_png, cleanup_svg },
 #endif
 #if CAIRO_HAS_BEOS_SURFACE
-	    { "beos", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR,
+	    { "beos", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR, 0,
 		create_beos_surface, cairo_surface_write_to_png, cleanup_beos},
-	    { "beos_bitmap", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR,
+	    { "beos-bitmap", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR, 0,
 		create_beos_bitmap_surface, cairo_surface_write_to_png, cleanup_beos_bitmap},
-	    { "beos_bitmap", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR_ALPHA,
+	    { "beos-bitmap", CAIRO_SURFACE_TYPE_BEOS, CAIRO_CONTENT_COLOR_ALPHA, 0,
 		create_beos_bitmap_surface, cairo_surface_write_to_png, cleanup_beos_bitmap},
 #endif
 
 #if CAIRO_HAS_DIRECTFB_SURFACE
-	    { "directfb", CAIRO_SURFACE_TYPE_DIRECTFB, CAIRO_CONTENT_COLOR,
+	    { "directfb", CAIRO_SURFACE_TYPE_DIRECTFB, CAIRO_CONTENT_COLOR, 0,
 		create_directfb_surface, cairo_surface_write_to_png, cleanup_directfb},
-	    { "directfb_bitmap", CAIRO_SURFACE_TYPE_DIRECTFB, CAIRO_CONTENT_COLOR_ALPHA,
+	    { "directfb-bitmap", CAIRO_SURFACE_TYPE_DIRECTFB, CAIRO_CONTENT_COLOR_ALPHA, 0,
 		create_directfb_bitmap_surface, cairo_surface_write_to_png,cleanup_directfb},
 #endif
 	};
@@ -1880,7 +1918,7 @@ cairo_test_expecting (cairo_test_t *test,
 	    }
 
 	    if (!found) {
-		fprintf (stderr, "Cannot test target '%.*s'\n", end - tname, tname);
+		fprintf (stderr, "Cannot test target '%.*s'\n", (int)(end - tname), tname);
 		exit(-1);
 	    }
 
@@ -1920,13 +1958,17 @@ cairo_test_expecting (cairo_test_t *test,
 		    _cairo_test_content_name (target->content),
 		    dev_offset);
 
+#ifdef HAVE_SIGNAL_H
 	    /* Set up a checkpoint to get back to in case of segfaults. */
 	    old_segfault_handler = signal (SIGSEGV, segfault_handler);
 	    if (0 == setjmp (jmpbuf))
+#endif
 		status = cairo_test_for_target (test, target, dev_offset);
+#ifdef HAVE_SIGNAL_H
 	    else
 	        status = CAIRO_TEST_CRASHED;
 	    signal (SIGSEGV, old_segfault_handler);
+#endif
 
 	    cairo_test_log ("TEST: %s TARGET: %s FORMAT: %s OFFSET: %d RESULT: ",
 			    test->name, target->name,
@@ -1984,6 +2026,11 @@ cairo_test_expecting (cairo_test_t *test,
 	}
     }
 
+    if (ret != CAIRO_TEST_SUCCESS)
+        printf ("Check %s%s out for more information.\n", test->name, CAIRO_TEST_LOG_SUFFIX);
+
+    /* if no target was requested for test, succeed, otherwise if all
+     * were untested, fail. */
     if (ret == CAIRO_TEST_UNTESTED)
 	ret = num_targets ? CAIRO_TEST_FAILURE : CAIRO_TEST_SUCCESS;
 
